@@ -2,6 +2,8 @@ from sqlalchemy import Column, String, Integer, Float
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.engine import create_engine
 from sqlalchemy.orm.session import Session
+import time
+import json
 
 Base = declarative_base()
 # generate tables
@@ -2122,6 +2124,55 @@ class SportVerify(Base):
         self.result = result
         self.score = score
 
+    @classmethod
+    def get_model_score(cls, path_json: str) -> dict:
+        """
+        检查表 verifies 最后5期的数据，得出所有模型的得分，并写入指定的json文件，并返回结果
+        base                         s1     1
+        verify == 'y' & 不包含开奖号  s2     1
+        verify == 'y' & 又包含开奖号  s3     2
+        verify == 'n'                       0
+        """
+        s1 = 1
+        s2 = 1
+        s3 = 2
+
+        sess = get_session_sport()
+        # 获取最后5期的期号
+        num_all = sess.query(SportNumber).order_by(SportNumber.issue).all()
+        num_last5 = num_all[-5:]
+        issue_begin = num_last5[0].issue
+
+        # 获取最后5期的 verifies 表数据
+        data_verifies = sess.query(cls).filter(cls.issue >= issue_begin).all()
+        dict_verifies = {x.inumabc: x for x in data_verifies}
+        result = dict()
+
+        # 循环模型名称
+        for num in range(10):
+            for p1 in range(10):
+                for p2 in range(10):
+                    for p3 in range(10):
+                        num_a_b_c = f'{num}{p1}{p2}{p3}'
+                        result[num_a_b_c] = s1
+                        # 循环最后5期的数据
+                        for n in num_last5:
+                            i = n.issue
+                            nums = [n.num1, n.num2, n.num3]
+                            i_num_a_b_c = f'{i}{num_a_b_c}'
+                            ins = dict_verifies.get(i_num_a_b_c)
+                            if ins.verify == 'y':
+                                if ins.num in nums:
+                                    result[num_a_b_c] = result[num_a_b_c] + s3
+                                else:
+                                    result[num_a_b_c] = result[num_a_b_c] + s2
+        # result
+        with open(path_json, 'w') as file_write:
+            json.dump(result, file_write)
+        print(">>> json result has been written at:", path_json)
+        return result
+
+
     @classmethod
     def add_score(cls, inumabc, issue, num, abc, result, score):
         sess = get_session_sport()
@@ -2147,6 +2198,54 @@ class SportVerify(Base):
         print(
             f">>> add score: inumabc: {inumabc}, issue: {issue}, num: {num}, abc: {abc}, result: {result}, score: {score}, verify: {ins.verify}")
 
+    @classmethod
+    def whether_needs_retrained(cls, num_abc: str, last_num: int, min_right: int):
+        """检查模型最后几期的准确率，判断是否合格
+
+        Args:
+            num_abc (str): num + abc [1008]
+            last_num (int): 检查最后几期
+            min_right (int): 最小正确数
+
+        Returns:
+            [type]: [description]
+        """
+        # 检查模型最后几期的准确率，判断是否合格
+        if min_right > last_num:
+            raise Exception(">>> ERROR PARAM: min_right > last_num")
+        if len(num_abc) != 4:
+            raise Exception(">>> ERROR PARAM: num_abc", num_abc)
+        num_abc = str(num_abc)
+        whether_train = True
+        num = num_abc[0]
+        sess = get_session_sport()
+        issue_all = SportNumber.get_issues()
+        issue_last = issue_all[-last_num:]
+        coll_verify = []
+        coll_verify_all = []
+        for i_target in issue_last:
+            i_num_abc = f'{i_target}{num_abc}'
+            v_ins = sess.query(cls).get(i_num_abc)
+            coll_verify.append(v_ins.verify)
+            temp_dict = {
+                'num123': f'{v_ins.num1}{v_ins.num2}{v_ins.num3}',
+                'result': v_ins.result,
+                'verify': v_ins.verify
+            }
+            coll_verify_all.append(temp_dict)
+            # print(f'model: {num_abc}, result: {temp_dict}')
+
+        # 检查 y 的数量
+        if coll_verify.count('y') >= min_right:
+            whether_train = False
+
+            # 检查 出号必y
+            for one_coll in coll_verify_all:
+                if one_coll['num123'].find(str(num)) > -1:
+                    if one_coll['verify'] == 'n':
+                        whether_train = True
+        print(f">>> whether_needs_retrained: {num_abc} - {whether_train}")
+        return whether_train
 
 class SportNumber5(Base):
     """ numbers@p3.db """
@@ -2453,7 +2552,67 @@ class SportVerifyX(Base):
     verify = Column(String(50))
 
     @classmethod
-    def add_score(cls, inumabc, issue, num, abc, result, score, channel):
+    def get_result_score_or_false(cls, issue_num_abc: str):
+        with get_session_sport() as sess:
+            # sess = get_session_sport()
+            data = sess.query(cls).get(issue_num_abc)
+            if data:
+                return {'label': data.result, 'predicts': data.score}
+            else:
+                return False
+
+    @classmethod
+    def get_model_score(cls, path_json: str) -> dict:
+        """
+        检查表 verifies 最后5期的数据，得出所有模型的得分，并写入指定的json文件，并返回结果
+        base                         s1     1
+        verify == 'y' & 不包含开奖号  s2     1
+        verify == 'y' & 又包含开奖号  s3     2
+        verify == 'n'                       0
+        """
+
+        channel = cls.__tablename__[-1]
+        s1 = 1
+        s2 = 1
+        s3 = 2
+
+        sess = get_session_sport()
+        # 获取最后5期的期号
+        num_all = sess.query(SportNumber5).order_by(SportNumber5.issue).all()
+        num_last5 = num_all[-5:]
+        issue_begin = num_last5[0].issue
+
+        # 获取最后5期的 verifies 表数据
+        data_verifies = sess.query(cls).filter(cls.issue >= issue_begin).all()
+        dict_verifies = {x.inumabc: x for x in data_verifies}
+        result = dict()
+
+        # 循环模型名称
+        for num in range(10):
+            for p1 in range(6):
+                for p2 in range(6):
+                    for p3 in range(6):
+                        num_a_b_c = f'{num}{p1}{p2}{p3}'
+                        result[num_a_b_c] = s1
+                        # 循环最后5期的数据
+                        for n in num_last5:
+                            i = n.issue
+                            nums = {'a': n.num1, 'b': n.num2, 'c': n.num3, 'd': n.num4, 'e': n.num5}.get(channel)
+                            i_num_a_b_c = f'{i}{num_a_b_c}'
+                            ins = dict_verifies.get(i_num_a_b_c)
+                            if ins.verify == 'y':
+                                if ins.num == nums:
+                                    result[num_a_b_c] = result[num_a_b_c] + s3
+                                else:
+                                    result[num_a_b_c] = result[num_a_b_c] + s2
+        # result
+        with open(path_json, 'w') as file_write:
+            json.dump(result, file_write)
+        print(">>> json result has been written at:", path_json)
+        return result
+
+    @classmethod
+    def add_score(cls, inumabc, issue, num, abc, result, score):
         """add score
 
         Args:
@@ -2465,6 +2624,9 @@ class SportVerifyX(Base):
             score (float): float result
             channel (str): 'a' or 'b' or 'c' or 'd' or 'e'
         """
+        channel = cls.__tablename__[-1]
+        if channel not in ['a', 'b', 'c', 'd', 'e']:
+            raise Exception(f">>> class: {cls.__class__.__name__}, method: add_score")
         sess = get_session_sport()
         ins = cls(inumabc)
         ins.issue = issue
@@ -2499,6 +2661,69 @@ class SportVerifyX(Base):
         print(
             f">>> {cls.__tablename__}: add score channel:{channel}: inumabc: {inumabc}, issue: {issue}, num: {num}, abc: {abc}, result: {result}, score: {score}, verify: {ins.verify}")
 
+    @classmethod
+    def whether_needs_retrained(cls, num_abc: str, last_num: int, min_right: int):
+        """检查模型最后几期的准确率，判断是否合格
+
+        Args:
+            num_abc (str): num + abc [1008]
+            last_num (int): 检查最后几期
+            min_right (int): 最小正确数
+
+        Returns:
+            [type]: [description]
+        """
+        # 检查模型最后几期的准确率，判断是否合格
+        if min_right > last_num:
+            raise Exception(">>> ERROR PARAM: min_right > last_num")
+        if len(num_abc) != 4:
+            raise Exception(">>> ERROR PARAM: num_abc", num_abc)
+        num_abc = str(num_abc)
+        whether_train = True
+        num = num_abc[0]
+        sess = get_session_sport()
+        issue_all = SportNumber5.get_issues()
+        issue_last = issue_all[-last_num:]
+        coll_verify = []
+        coll_verify_all = []
+        for i_target in issue_last:
+            i_num_abc = f'{i_target}{num_abc}'
+            # print("i_num_abc: ", i_num_abc)
+            v_ins = sess.query(cls).get(i_num_abc)
+            coll_verify.append(v_ins.verify)
+            if cls.__tablename__ == 'verifya':
+                temp_num123 = str(v_ins.num1)
+            elif cls.__tablename__ == 'verifyb':
+                temp_num123 = str(v_ins.num2)
+            elif cls.__tablename__ == 'verifyc':
+                temp_num123 = str(v_ins.num3)
+            elif cls.__tablename__ == 'verifyd':
+                temp_num123 = str(v_ins.num4)
+            elif cls.__tablename__ == 'verifye':
+                temp_num123 = str(v_ins.num5)
+            else:
+                raise(">>> ERROR TABLENAME")
+            temp_dict = {
+                'num123': temp_num123,
+                'result': v_ins.result,
+                'verify': v_ins.verify
+            }
+            coll_verify_all.append(temp_dict)
+            print(f'model: {num_abc}, result: {temp_dict}')
+
+        # 检查 y 的数量
+        if coll_verify.count('y') >= min_right:
+            whether_train = False
+
+            # 检查 出号必y
+            for one_coll in coll_verify_all:
+                if one_coll['num123'] == str(num):
+                    if one_coll['verify'] == 'n':
+                        whether_train = True
+
+        print(f">>>Table: {cls.__tablename__} whether_needs_retrained: {num_abc} - {whether_train}")
+        return whether_train
+
 
 class SportVerifyA(SportVerifyX):
     __tablename__ = "verifya"
